from django.conf.urls.defaults import *

urlpatterns = patterns('wellknown.views',
    url(r'^\.well-known/(?P<path>.*)', 'handle', name='wellknown'),
    url(r'^crossdomain\.xml$', 'crossdomain', name='crossdomain.xml'),
    url(r'^robots\.txt$', 'robots', name='robots.txt'),
)



from django.db import models
from django.db.models.signals import post_save
import mimetypes
import wellknown

#
# resource model
#

class Resource(models.Model):
    path = models.CharField(max_length=128)
    content = models.TextField(blank=True)
    content_type = models.CharField(max_length=128, blank=True)
    
    class Meta:
        ordering = ('path',)
    
    def __unicode__(self):
        return self.path
    
    def save(self, **kwargs):
        self.path = self.path.strip('/')
        if not self.content_type:
            self.content_type = mimetypes.guess_type(self.path)[0] or 'text/plain'
        super(Resource, self).save(**kwargs)

#
# update resources when models are saved
#

def save_handler(sender, **kwargs):
    reg = kwargs['instance']
    wellknown.register(
        reg.path,
        content=reg.content,
        content_type=reg.content_type,
        update=True
    )

post_save.connect(save_handler, sender=Resource)



from django.template.loader import render_to_string
import mimetypes

from wellknown.resources import HostMeta

_cache = { }

def get_hostmeta():
    return get_resource('host-meta')[0]

def get_resource(path):
    return _cache.get(path, (None, None))

def register(path, handler=None, template=None, content=None, content_type=None, update=False):
    
    if path in _cache:
        raise ValueError(u"duplicate resource for %s" % path)
        
    if content_type is None and not update:
        content_type = mimetypes.guess_type(path)[0] or 'text/plain'
        
    if handler:
        _cache[path] = (handler, content_type)
    elif template:
        _cache[path] = (render_to_string(template), content_type)
    elif content:
        _cache[path] = (content, content_type)
    else:
        raise ValueError(u"either handler, template, or content must be specified")

__all__ = ['register','get_resource','get_hostmeta']

# register default host-meta handler.
register('host-meta', handler=HostMeta(),
         content_type='application/xrd+xml')



from django.conf import settings
from django.http import HttpResponse, Http404
import wellknown

try:
    from robots.views import rules_list
except ImportError:
    rules_list = None

def handle(request, path, *args, **kwargs):
    """ Basic handler view to either display cached content
        or make call to actual handler method for rendering.
    """

    (handler_or_content, content_type) = wellknown.get_resource(path)

    if handler_or_content is None:
        raise Http404()

    if callable(handler_or_content):
        content = handler_or_content(request, content_type=content_type, *args, **kwargs)
    else:
        content = handler_or_content

    return HttpResponse(content or '', content_type=content_type)

def crossdomain(request, *args, **kwargs):
    """ View that overrides /crossdomain.xml to
        handle as a well-known resource.
    """
    return handle(request, 'crossdomain.xml', *args, **kwargs)

def robots(request, *args, **kwargs):
    """ Handle /robots.txt as a well-known resource or
        pass off request to django-robots.
    """
    if rules_list:  # use django-robots if it is installed
        return rules_list(request, *args, **kwargs)
    return handle(request, 'robots.txt', *args, **kwargs)


import re

from django.conf import settings
from django.template.loader import render_to_string
from xrd import XRD, Link, Element

LANGUAGE_CODE = getattr(settings, "LANGUAGE_CODE", "en-us")

class HostMeta(XRD):
    
    def __init__(self):
        
        super(HostMeta, self).__init__()
        
        self.attributes.append(('xmlns:hm','http://host-meta.net/ns/1.0'))
        
        hosts = getattr(settings, "WELLKNOWN_HOSTMETA_HOSTS", ('example.com',))
        for host in hosts:
            self.elements.append(Element('hm:Host', host))
        
    def __call__(self, *args, **kwargs):
        doc = self.to_xml()
        xml = doc.toprettyxml(indent='  ')
        expr = re.compile(r'>\n\s+([^<>\s].*?)\n\s+</', re.DOTALL)
        return expr.sub(r'>\g<1></', xml)
